/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu");

dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.ResourceLink");

dojo.require("dm.analysis.app.ui.DiagramUtilityMethods");
var DiagramUtilityMethods = dm.analysis.app.ui.DiagramUtilityMethods;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");

var ResourceLinkDivName = "resourceLinkDiv";

/**
 * 
 */
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu", null, {
	
	// Constructor arguments
	editor: null,		// underlying analysis diagram editor
	diagram: null,		// the instance of ImpactAnalysisDiagram
		
	// Define the menu actions for Nodes and Links
	_nodeMenuActions: null,
	_linkMenuActions: null,
	_popupMenu: null,
	
	// Keep a list of the resource links for cleanup.
	_resourceLinks: null,
	
	constructor: function(args) {
		dojo.mixin(this, args);
		
		// Initialize the menus for Nodes and Links
		this._initNodeMenu();
		this._initLinkMenu();
		this._resourceLinks = [];
		
		// Create an instance of the popup menu.
		this._popupMenu = new dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu_Menu();
	},
	
	/** Perform the required clean up when the diagram is
	 *  recreated. When the diagram nodes and links are
	 *  destroyed, the existing resource links need to be
	 *  deleted and the pop-up menu must be closed.
	 */
	cleanup: function() {
		// Close the popup menu if it happens to be showing.
		this._popupMenu.closeMenu();
		
		// Destroy all of the resource links
		dojo.forEach(this._resourceLinks, function(resourceLink, i) { resourceLink.destroy(); });
		this._resourceLinks = [];
		
		// Remove all of the special 'div" resource link nodes from the diagram.
		var diagramDomNode = this.diagram.domNode;
		dojo.query(('.' + ResourceLinkDivName), diagramDomNode).forEach(function(divNode, index) {
			diagramDomNode.removeChild(divNode);	
		});
		
	},
	
	/** When the analysis popup menu is destroyed, perform
	 *  the necessary clean up.
	 */
	destroy: function() {
		this.cleanup();
		this.inherited(arguments);
	},
	
	/** Show the popup menu for the given analysis node.
	 *
	 *  All nodes on the diagram have the same set of actions. 
	 *  However, the disable callback can be used to choose
	 *  which actions are applicable for a given node.
	 *
	 * @param node the graph node
	 */
	showNodePopupMenu: function(node) {
		// Create the resource link widget for the preview action and
		// get the special 'div" node to position the resource link widget.
		// The resource link must be created before the pop-up menu is 
		// initialized or else the preview action will be disabled.
		var resourceLinkDiv = this._createPreviewResourceLink(node);
		
		// Initialize the popup menu for this node.
		this._popupMenu.initialize(node, this._nodeMenuActions);
		
		// Determine the coordinates for the popup menu.
		// The bounds for the node are relative to its ilog diagram.
		// Adjust the coordinates for the actual screen
		var viewport = this.diagram.impactDiagram.getViewport();
		var bounds = node.getBounds(null);
		var nodePos = viewport.pointContentToClient(bounds);
		var diagramPos = dojo.position(this.diagram.impactDiagram.domNode);
		var topLeft_x = Math.round(nodePos.x + diagramPos.x);
		var topLeft_y = Math.round(nodePos.y + diagramPos.y);
		
		// Set the position of the preview 'div" node to the coordinates 
		// of the top left corner of the graph element.
		if (resourceLinkDiv) {
			dojo.style(resourceLinkDiv, { "left": topLeft_x+'px', "top": topLeft_y+'px' });
		}
		
		// Now show the actual menu
		this._popupMenu.showMenu({
			x: topLeft_x,
			y: topLeft_y - 20	// 20 = approximate height of toolbar that has 16x16 icons
		});
	},
	
	/** Determine if the popup menu is being shown for the given graph element.
	 *
	 * @return true = menu is being shown for the given element
	 */
	isMenuShowingForElement: function(graphElement) {
		return this._popupMenu.isMenuShowingForElement(graphElement); 
	},
	
	/** Hide the popup menu. 
	 */
	hidePopupMenu: function() {
		this._popupMenu.closeMenu();
	},
	
	/** Initialize the pop-up menu actions for analysis Nodes.
	 */
	_initNodeMenu: function() {
		this._nodeMenuActions = new dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu_MenuActions();
		
		// Add the Preview action to the menu.
		this._nodeMenuActions.addAction( {
			label: messages.actionPreview,
			iconFile: "graphics/icons/preview.gif",
			disableCallback: dojo.hitch(this, this._disablePreviewCallback),
			actionCallback:  dojo.hitch(this, this._previewCallback)
		} );
		
		// Add the Remove action to the menu.
		this._nodeMenuActions.addAction( {
			label: messages.actionRemove,
			iconFile: "graphics/icons/remove_grey.gif",
			disableCallback: dojo.hitch(this, this._disableRemoveCallback),
			actionCallback:  dojo.hitch(this, this._removeCallback)
		} );
			
	},

	/** Initialize the pop-up menu actions for analysis Links.
	 */
	_initLinkMenu: function() {
		this._linkMenuActions = null;	// not supported yet
	},
	
	/** Determine if the Remove action is to be disabled. 
	 *  The target resource of the analysis cannot be removed
	 *  and neither can elements that were added.
	 *
	 * @param graphElement the Node or Link that for the Remove action
	 * @return true (disable) if the element is the target analysis resource
	 */
	_disableRemoveCallback: function(graphElement) {
		var data = graphElement.getData();
		var uri  = data.item.URI;
		var disable = (uri == this.editor.editorInput.impactAnalysis.targetResourceUri);
		
		if (!disable) {
			for (var tmpUri in  this.editor.editorInput.impactAnalysis.addedElements) {
				if (uri == tmpUri) {
					disable = true;
					break;
				}
			}
		}
		return disable;
	},
	
	/** Callback to remove an element from the diagram.
	 */
	_removeCallback: function(graphElement, event) {
		var data = graphElement.getData();

		if (data.item.URI.length != 1){
			console.warn("For removal, exactly one element should be selected. " + data.item.URI.length + " selected instead.");
			return;
		}

		var uri  = data.item.URI[0];
		var name = data.item.name[0];
		var type = (data.item.type && data.item.type[0]) ? data.item.type[0] : "";
		
		// Determine the type of the resource
		var split;
		if (type.indexOf("/") > 0) {
			split = type.split("/");
			type = split[split.length-1];
		}
		if (type.indexOf("#") > 0) {
			split = type.split("#");
			type = split[split.length-1];
		}

		// Confirm with the user before removing the element					
		if (confirm(messages.confirmDeleteElement)) {
			this.diagram.removeElement(uri, name, type);
		}
	},
	
	
	/** Determine if the Preview action is to be disabled. 
	 *  The preview can only be performed if there is a
	 *  URI for compact rendering (e.g., the resource link
	 *  was created for the graph element).
	 *
	 * @param graphElement the Node or Link that for the Preview action
	 * @return true (disable) if the resource does not have a compact rendering
	 */
	_disablePreviewCallback: function(graphElement) {
		var data = graphElement.getData();
		var disable = !data._resourceLink;
		return disable;
	},
	
	/** Callback to preview a resource.
	 */
	_previewCallback: function(graphElement, event) {
		// Get the resource link widget from the graph element
		var data = graphElement.getData();
		var resourceLink = data._resourceLink;
		
		// Show the preview action compact rendering.
		if (resourceLink) {
			setTimeout(dojo.hitch(resourceLink, function() {
				if (this.popup && !this.popup._isOpen) {
					this.popup.open();
				}
			}), 10);
		}
	},
	
	/** Create the preview resource link widget. The resource
	 *  link widget will position itself near to its parent
	 *  node. To accomplish this, a special div node is 
	 *  created as a child of the graph element's raw node
	 *  but it is will be given an absolute position on the
	 *  page just before the pop-up menu is displayed.
	 *
	 * @param graphElement the analysis Node or Link.
	 * @return the 'div" node for positioning the resource link widget (or null)
	 */
	_createPreviewResourceLink: function(graphElement) {
		var data = graphElement.getData();
		
		// If the resource link has not been created yet
		if (!data._resourceLink) {
			// Compute the URI for rendering the resource information
			var resourceUri = data.item.URI[0];
			var renderingUri = DiagramUtilityMethods.call_getCompactRenderingUri(resourceUri);
		
			// Compute the presentation URI to open the resource
			var presentationUri = DiagramUtilityMethods.call_getResourceEditUri(resourceUri);
			if (!presentationUri) {
				presentationUri = resourceUri;
			}
			
			// If the resource does not have a URI for compact rendering
			if (!renderingUri) {
				// Disable the preview action for this resource
				data._resourceLink = null;
				data._resourceLinkDiv = null;
			}
			
			else {		
				// Create a <div> node that will be the parent of the 
				// resource link widget. This div node and the resource
				// link are created once per graph element but the div
				// node will be positioned each time the pop-up menu
				// is displayed to absolute coordinates.
				var domNode = this.diagram.domNode;
				var div = document.createElement("div");
				dojo.style(div, {
					"visibility": "hidden",
					"display":    "block",
					"position":   "fixed",
					"width":      "5px",
					"height":     "5px"
				});
				div.className = ResourceLinkDivName;
				domNode.appendChild(div);
				
				// Use our own 'srcNodeRef' so that we can set a target
				// which opens the link in a new window/tab
				var link = dojo.create('a', {
					href: presentationUri,
					target: '_blank'
				});

				// Create the resource link
				var resourceLink = new jazz.ui.ResourceLink({
					uri: renderingUri,
					presentationUri: presentationUri,
					lazyFetch : true,
					label: "-"
				}, link);
				div.appendChild(resourceLink.domNode);
				resourceLink.startup();
		
				// Save the resource link widget
				data._resourceLink = resourceLink;
				data._resourceLinkDiv = div;
				this._resourceLinks.push(resourceLink);
			}
		}

	// Return the 'div' node to be positioned
	return data._resourceLinkDiv;
	}
	
});

/** Define the object for specifying menu actions and cascading menu actions.
 *  
 *  Before the pop-up menu can be displayed, the actions for the menu must be defined.
 *  Each instance of this class represents a single menu and its actions. Cascade
 *  menus are not supported.
 *
 *  To use this class, create an instance and call addAction() for each menu item. 
 *  The "add" method requires a label and an action callback function. The arguments
 *  can also include an icon file, and a callback function to determine if the action 
 *  should be disabled for the corresponding graph element.
 */
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu_MenuActions", null, {

	// The actions defined for this menu (in order)
	_actions: null,
	
	constructor: function() {
		this._actions = [];
		this.inherited(arguments);
	},
	
	/** Add an action to the menu.
	 *
	 * @param label				the label for the action (required)
	 * @param iconFile			the icon file name (relative to this web plug-in)
	 * @param actionCallback	required function(graphElement, event) that is called when action is clicked
	 * @param disableCallback	optional function(graphElement) that is called to determine if action should
	 *							be disabled for the graph element
	 */
	addAction: function(args) {
		// An action must at least define a label and the action callback
		if (args.label && dojo.isFunction(args.actionCallback)) {
			var action = {
				label: args.label,
				actionCallback: args.actionCallback
			};
			if (args.iconFile) {
				action.iconFile = args.iconFile;
			}
			if (dojo.isFunction(args.disableCallback)) {
				action.disableCallback = args.disableCallback;
			}
			this._actions.push(action);
		}
	},

	/** Return the list (an array) of actions for this menu.
	 *
	 *  The following fields may be defined for each action object: 
	 *		label			- must be defined
	 *		iconFile		- optionally defined
	 *		disableCallback	- optionally defined
	 *		actionCallback	- must be defined for an action
	 *
	 * @return an array of action objects
	 */
	getActions: function() {
		return this._actions;
	}
	  
});

/** Define the actual action menu widget. Only one
 *  instance of this class is needed because only one pop-up menu
 *  should be active at any time. 
 */
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu_Menu", null, {

	// The menu and its attributes
	_menu: null,
	_menuListeners: [],
	_menuButtons: [],
	
	// Remember the corresponding graph element for the menu 
	_menuOnElement:  null,
	
	constructor: function() {
		this._menu = null;
	},
	
	/** Determine if the popup menu is being shown for the given graph element.
	 *
	 * @return true = menu is being shown for the given element
	 */
	isMenuShowingForElement: function(graphElement) {
		return graphElement === this._menuOnElement; 
	},
	
	/** Initialize the menu for the given ilog GraphElement (Node or Link).
	 *  The menu actions should be an instance of the MenuActions class that
	 *  define the pop-up menu for the given graph element (Node or Link)
	 */
	initialize: function(graphElement, menuActions) {
		// If a menu already exists, close it first (can only have one menu at any time)
		if (this._menu) {
			this.closeMenu();
		}
		
		// Create the new menu (toolbar) and add the actions to the menu.
		this._menu = new jazz.ui.SimpleToolbar();
		dojo.addClass(this._menu.domNode, "analysisPopupMenu");
		
		// Create the actions for the menu (toolbar)
		var actions = menuActions.getActions();
		var length  = actions.length;
		for (var i = 0; i < length; i++) {
			this._createAction(this._menu, graphElement, actions[i]);
		}	
		this._menu.startup();
		
		// Remember the graph element for which the menu is being displayed
		this._menuOnElement = graphElement;		
	}, 
	
	/** Create one action for a given menu. 
	 *
	 * @param menu  the corresponding menu widget for the actions
	 * @param graphElement the Node or Link that the menu works on
	 * @param action the action definition
	 */	
	_createAction: function(menu, graphElement, action) {
		var disabled = false;
		if (action.disableCallback) {
			disabled = action.disableCallback(graphElement);
		}
		
		// Compute the URL for the button image
		var imageUrl = null;
		if (action.iconFile) {
			imageUrl = dojo.moduleUrl("dm.analysis.app", action.iconFile);   // TDB needs a constant for app ID
		}
			
		// Compute the button callback function
		var buttonCallback = dojo.hitch(this, function(e) {
			action.actionCallback(graphElement, e);
			this.onMenuClicked();
		});
		
		// Define the args for the action button
		var args = {
			label: action.label,
			showLabel: (imageUrl ? false : true)
		};
		if (disabled) {
			args.className = "jazz-ui-SimpleToolbar-button-disabled";
		}
		if (imageUrl) {
			args.imgSrc = imageUrl;
		}
	
		// Create the button for the action (show label if no icon)
		var button = menu.addButton(this, buttonCallback, args);
		dojo.addClass(button.domNode, "analysisPopupMenuButton");
		if (!disabled) {
			this._registerMenuButton(button);
		}
	},
	
	/** Register the action button. As part of registration,
	 *  the button is added to the focusable button list
	 *  and key press events are monitored for escape and the
	 *  left/right arrow keys. The underlying Jazz simple toolbar
	 *  only support <tab> traversal between buttons, but when
	 *  the toolbar is in a popup menu, the <tab> moves the
	 *  focus away from the toolbar, which automatically closes.
	 *
	 * @param button	the Jazz simple toolbar button
	 */
	_registerMenuButton: function(button) {
		var buttonIndex = this._menuButtons.length;
		this._menuButtons.push(button);

		this._menuListeners.push(dojo.connect(button.domNode, "onkeydown", this, function(event) {
			var k = dojo.keys;
			var index = buttonIndex;
			if (event.keyCode === k.ESCAPE) {
				this.closeMenu();
			} 
			
			else if (event.keyCode === k.LEFT_ARROW) {
				index--;
				if (index < 0) {
					index = this._menuButtons.length - 1;
				}
				this._menuButtons[index].domNode.focus();
			}
			
			else if (event.keyCode === k.RIGHT_ARROW) {
				index++;
				if (index >= this._menuButtons.length) {
					index = 0;
				}
				this._menuButtons[index].domNode.focus();
			}
		}));
	},
	
	/** Close the action menu 
	 */
	closeMenu: function() {
		if (this._menu) {
			// Remove all of the listeners
			dojo.forEach(this._menuListeners, function(listener, i) { dojo.disconnect(listener); });
			this._menuListeners = [];
			
			// Destroy the entire menu
			dijit.popup.close(this._menu);
			this._menu.destroyRecursive();
			delete this._menu;
			this._menuButtons = [];
			this._menuOnElement = null;
		}
	},
	
	/** When a normal menu item is clicked, close the menu.
	 */
	onMenuClicked: function() {
		this.closeMenu();
	},
	
	/** Show the pop-up menu at the given coordinates.
	 *
	 * @param coords the x and y position for the menu
	 */
	showMenu: function(coords) {
		if (this._menu) {
			dijit.popup.open({
				popup: this._menu,
				x: coords.x,
				y: coords.y,   // icons typically 16x16 pixels
				orient: 'L',
				onExecute: dojo.hitch(this, this.closeMenu),
				onCancel:  dojo.hitch(this, this.closeMenu)
			});
			this._menuButtons[0].domNode.focus();	
		}
	}
});

})();
